'use strict';

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _chaiAsPromised = require('chai-as-promised');

var _chaiAsPromised2 = _interopRequireDefault(_chaiAsPromised);

var _sinon = require('sinon');

var _sinon2 = _interopRequireDefault(_sinon);

var _sinonChai = require('sinon-chai');

var _sinonChai2 = _interopRequireDefault(_sinonChai);

var _index = require('../index');

var _index2 = _interopRequireDefault(_index);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_chai2.default.use(_chaiAsPromised2.default);
_chai2.default.use(_sinonChai2.default);

var expect = _chai2.default.expect;

var fail = function fail(x) {
  return function () {
    return Promise.reject(x);
  };
};
var pass = function pass(x) {
  return function () {
    return Promise.resolve(x);
  };
};

describe('fallback-plan', function () {
  describe('fallback', function () {
    it('returns a single resolved Promise\'s value', function () {
      return expect(_index2.default.fallback([pass(42)], Promise)).to.eventually.equal(42);
    });
    it('returns a single rejected Promise\'s reason', function () {
      return expect(_index2.default.fallback([fail(31)], Promise)).to.be.rejectedWith(31);
    });
    it('returns the first resolved Promise\'s value', function () {
      return Promise.all([expect(_index2.default.fallback([fail(1), pass(2)], Promise)).to.eventually.equal(2), expect(_index2.default.fallback([fail(1), fail(2), pass(3)], Promise)).to.eventually.equal(3), expect(_index2.default.fallback([pass(1), pass(2), pass(3)], Promise)).to.eventually.equal(1), expect(_index2.default.fallback([fail(1), pass(2), fail(3)], Promise)).to.eventually.equal(2)]);
    });
    it('returns the last rejected Promise\'s value', function () {
      return Promise.all([expect(_index2.default.fallback([fail(1), fail(2)], Promise)).to.be.rejectedWith(2), expect(_index2.default.fallback([fail(1), fail(2), fail(3)], Promise)).to.be.rejectedWith(3)]);
    });
    it('only requires a single resolved Promise to resolve', function () {
      return Promise.all([expect(_index2.default.fallback([pass(1), fail(2), fail(3), fail(4)], Promise)).to.eventually.equal(1), expect(_index2.default.fallback([fail(1), pass(2), fail(3), fail(4)], Promise)).to.eventually.equal(2), expect(_index2.default.fallback([fail(1), fail(2), pass(3), fail(4)], Promise)).to.eventually.equal(3), expect(_index2.default.fallback([fail(1), fail(2), fail(3), pass(4)], Promise)).to.eventually.equal(4)]);
    });
    it('supports values in the input list', function () {
      return Promise.all([expect(_index2.default.fallback([42])).to.eventually.equal(42)]);
    });
    it('supports functions in the input list', function () {
      return Promise.all([expect(_index2.default.fallback([function () {
        return 42;
      }])).to.eventually.equal(42), expect(_index2.default.fallback([function () {
        return function () {};
      }])).to.eventually.be.a('function')]);
    });
    it('supports Promises in the input list', function () {
      return Promise.all([expect(_index2.default.fallback([Promise.resolve(42)])).to.eventually.equal(42)]);
    });
    it('supports functions that throw and treats them like a rejection', function () {
      return Promise.all([expect(_index2.default.fallback([function () {
        throw new Error();
      }])).to.be.rejected]);
    });
  });

  describe('retry', function () {
    it('supports multiple retries up to a given number', function () {
      var badSpy = _sinon2.default.stub().returns(Promise.reject(2));
      return _index2.default.retry(badSpy, 5, Promise).catch(function () {
        expect(badSpy).to.have.callCount(5);
      });
    });

    it('supports unbounded multiple retries', function () {
      var countdown = 20;
      var multiSpy = _sinon2.default.spy(function () {
        countdown -= 1;
        if (countdown) {
          return Promise.reject(countdown);
        }
        return Promise.resolve(100);
      });
      return _index2.default.retry(multiSpy, 0, Promise).then(function (value) {
        expect(multiSpy).to.have.callCount(20);
        expect(value).to.equal(100);
      });
    });
  });

  describe('cycle', function () {
    it('cycles through sets of parameters for the same function', function () {
      return Promise.all([expect(_index2.default.cycle([[1], [2], [3]], function (x) {
        return Promise.resolve(x);
      })).to.eventually.equal(1), expect(_index2.default.cycle([[Promise.reject(1)], [Promise.reject(2)], [Promise.resolve(3)]], function (x) {
        return x;
      })).to.eventually.equal(3)]);
    });
  });
});